/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import <AppKit/AppKit.h>
#import "IDLNameService.h"
#import "IDLInterface_Private.h"
#import "IDLDefinitionScope.h"
#import "IORView.h"
#import "CORBAObject_Private.h"
#import "IDLGIOPVersion.h"
#import "ORB_Private.h"
#import "BindingItem.h"
#import "IDLException.h"
#import "IDLExceptionController.h"

@implementation IDLNameService

static id _instance = nil;
static NSString* VXHostAddress(NSHost* host);
//NSString* VXUpdateObject = @"CORBAObjectUpdated";

// Cannot use standard saving frame in the defaults
- (void) _setFrame
{
    NSString* str = [[NSUserDefaults standardUserDefaults]stringForKey:@"NameServiceFrame"];
    if(str){
        NSScanner* scanner = [NSScanner scannerWithString:str];
        NSPoint p;
        NSRect r = [window frame];
        [scanner scanFloat:&p.x];
        [scanner scanFloat:&p.y];
        r.origin.y = p.y - r.size.height;
        [window setFrame:r display:NO];
    }
}

- (void) _saveFrame
{
    NSRect r = [window frame];
    NSPoint p = {r.origin.x, r.origin.y + r.size.height};
    NSString* str = [NSString stringWithFormat:@"%f %f %f %f", p.x,p.y,r.size.width, r.size.height];
    [[NSUserDefaults standardUserDefaults]setObject:str forKey:@"NameServiceFrame"];
}

- (void) _setControls
{
    @try{
		int cnt = [[NSUserDefaults standardUserDefaults]integerForKey:@"nameServiceFetchCount"];
		CORBAObject* obj = [_interface interfaceConnection];
		if(cnt <=0)
			cnt = 100;
		[listCountField setIntValue:cnt];
		if(obj){
			NSString* hostName = [obj host];
			NSNumber* port = [obj port];
			NSHost* host = [hostName length] > 0? [NSHost hostWithName:hostName] : [NSHost currentHost];
			NSString* addr = VXHostAddress(host);
			[hostField setStringValue:addr?addr:@""];
			[portField setIntValue:[port unsignedShortValue]];
		}else{
			NSHost* host = [NSHost currentHost];
			NSString* addr = VXHostAddress(host);
			[hostField setStringValue:addr?addr:@""];
			[portField setIntValue:0];
		}

	}@catch(NSException *e){
    
    }
}

- init
{
    self = [super init];
    [NSBundle loadNibNamed:@"NameService" owner:self];
    
    _interface = [(NSObject*)[[IDLDefinitionScope rootScope]definitionForNamePath:@"CosNaming/NamingContext"]retain];
    
    [[NSNotificationCenter defaultCenter]addObserver:self
            selector:@selector(invalidateTableItem:)
            name:VXInvalidateTableItem
            object:nil];
    [self _setControls];
    [table setDoubleAction:@selector(_resolveItem:)];
    return self;
}

- (void) awakeFromNib
{
    NSView* view = [[connectBox superview]retain];
    [view removeFromSuperview];
    [(NSWindow*)window setContentView:view];
    [view release];
    //[window setFloatingPanel:YES];
    [window setShowsResizeIndicator:NO];
    [tableBox retain];
    [tableBox removeFromSuperview];
    [messageField setStringValue:@"No selection"];
    [self _setFrame];
}

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [window release];
    [_interface release];
    [tableBox release];
    [_dataSource release];
    [super dealloc];
}

+ nameService
{
    if(!_instance)
        _instance = [[self alloc]init];
    return _instance;
}

- (void) setDataSource:src
{
	if(src != _dataSource){
		[_dataSource release];
		_dataSource = [src retain];
	}
}

- (void) listEntries
{
    @try{
		if(!_dataSource){
			[self setDataSource:[[[BindingItem allocWithZone:[self zone]]initRoot:[_interface interfaceConnection]]autorelease]];
			[_dataSource fetch];
			[table setDataSource:_dataSource];
			[table reloadData];
		}
		[self showTableView];
    }@catch(NSException *localException){
		[self setDataSource:nil];
		[table setDataSource:nil];
		[table reloadData];
		NSLog(@"%@: %@\n", [localException name], [localException reason]);
		[self presentException:localException];
    }
}

- (void) reloadList:sender
{
    @try{
		[_dataSource fetch];
		[table reloadData];
		[table deselectAll:self];
    }@catch(NSException *localException){
		[self presentException:localException];
    }
}

- (void) reloadContextList:sender
{
    int row = [table selectedRow];
    if(row >= 0){
        BindingItem* item = [table itemAtRow:row];
        if([item isResolved] && [item isContext]){
            @try{
				[item fetch];
				[table reloadItem:item reloadChildren:YES];
            }@catch(NSException *localException){
				[self presentException:localException];
            }
        }else{
            NSBeep();
            if(![item isContext])
                [messageField setStringValue:@"Not a context. Select context or unselect all"];
            else
                [messageField setStringValue:@"Not resolved. Resolve first."];
        }
    }else
        [self reloadList:sender];
}

- (void) invalidateTableItem:(NSNotification*)notification
{
    BindingItem* item = [notification object];
    if(item != _dataSource){
        if([table isItemExpanded:item])
            [table collapseItem:item];
        [table reloadItem:item];
    }else{
		CORBAObject* object = [item connection];
		if(object){
			// The item's conection object is changed by location forward exception
			// We have to retain the old one
			//[_interface setInterfaceConnection:object]; // Don't do this
			//[hostField setStringValue:[object host]];
			//[portField setIntValue:[[object port]unsignedShortValue]];
			[iorView setNeedsDisplay:YES];
		}else{
			[table setDataSource:nil];
			[self setDataSource:nil];
			[self hideTableView];
			[iorView setNeedsDisplay:YES];
		}
        [table reloadData];
	}
}

- (void) connect:sender
{
    unsigned short port = [portField intValue];
    NSString* hostName = [hostField stringValue];
    const char* svc = "NameService";
    NSData* key = [NSData dataWithBytes:svc length:strlen(svc)];
    NSArray* version = [[giopPopUp title]componentsSeparatedByString:@"."];
    IDLGIOPVersion* giopVersion = [[[IDLGIOPVersion alloc]
        initVersion:[[version objectAtIndex:0]intValue]:[[version objectAtIndex:1]intValue]]
            autorelease];
    id naming = [[[CORBAObject allocWithZone:[ORB zone]]initWithInterface:_interface]autorelease];
    [naming addProfileWithKey:key host:hostName port:port version:giopVersion];
    [_interface setInterfaceConnection:naming];
	[iorView setConnection:naming];
//    [[NSNotificationCenter defaultCenter]postNotificationName:VXUpdateObject object:nil];
    [self listEntries];
}

- (void) list:sender
{

}

- (IDLInterface*) interface
{
	return _interface;
}

- (BOOL) isConnected
{
    return [_interface interfaceConnection] != nil;
}

- (void) orderFront:sender
{
    [window makeKeyAndOrderFront:self];
    if([self isConnected])
        [self listEntries];
}

- (void) invalidateTable
{
    [table reloadData];
}

- (void) hideTableView
{
    if([tableBox superview]){
        NSRect f = [window frame];
        NSRect r1 = [tableBox frame];
        NSSize s = [window minSize];
        f.size.height -= r1.size.height;
        f.origin.y += r1.size.height;
        s.height = f.size.height;
        [[window contentView]setAutoresizesSubviews:NO];
        [window setFrame:f display:YES animate:YES];
        [tableBox removeFromSuperview];
        [[window contentView]setAutoresizesSubviews:YES];
        [window setMinSize:s];
        [window setShowsResizeIndicator:NO];
        s.width += 120;
        [window setMaxSize:s];
    }
}

- (void) showTableView
{
    if(![tableBox superview]){
        NSRect f = [window frame];
        NSRect r0 = [connectBox frame];
        NSRect r1 = [tableBox frame];
        NSSize s = [window minSize];
        r1.size.width = r0.size.width;
        r1.origin.x = r0.origin.x;
        r1.origin.y = r0.origin.y + r0.size.height;
        
        f.origin.y -= r1.size.height;
        f.size.height += r1.size.height;
        s.height += 150;
        [connectBox setFrame:r0];
        [tableBox setFrame:r1];
        [[window contentView]addSubview:tableBox];
       // [filesBox setAutoresizingMask:NSViewWidthSizable|NSViewMinYMargin];
        [[window contentView]setAutoresizesSubviews:NO];
        [window setFrame:f display:YES animate:YES];
        [window display];
        [[window contentView]setAutoresizesSubviews:YES];
        [window setMinSize:s];
        [window setShowsResizeIndicator:YES];
        s.width += 120;
        s.height += 250;
        [window setMaxSize:s];
    }
}

- (void)outlineView:(NSOutlineView *)outlineView 
        willDisplayCell:(id)cell 
        forTableColumn:(NSTableColumn *)tableColumn 
        item:(id)item
{
    NSString* identifier = [tableColumn identifier];
    if([identifier isEqualToString:@"object"]){
        [cell setFont:[NSFont boldSystemFontOfSize:10]];
        if([item isResolved])
            [cell setTextColor:[NSColor blackColor]];
        else
            [cell setTextColor:[NSColor redColor]];
    
    }else{
        [cell setFont:[NSFont systemFontOfSize:10]];
        [cell setTextColor:[NSColor blackColor]];
        
    }
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
    int row = [table selectedRow];
    BindingItem* item = (row>=0)?[table itemAtRow:row]:nil;
    if([item isResolved] || item == nil){
        if([resolveButton isEnabled])
            [resolveButton setEnabled:NO];
        if(item){
            NSString* typeId = [item objectTypeId];
            [messageField setStringValue:typeId?typeId:@"Undefined"];
        }else
            [messageField setStringValue:@"No selection"];
    }else{
        if(![resolveButton isEnabled])
            [resolveButton setEnabled:YES];
        [messageField setStringValue:@"Not resolved"];
    }
}

- (void)windowDidMove:(NSNotification *)aNotification
{
    [self _saveFrame];
}
- (void)windowDidResize:(NSNotification *)aNotification
{
    [self _saveFrame];
}

- (void) click:sender
{

}

- (void) resolveItem:sender
{
    int row = [table selectedRow];
    if(row >= 0){
        BindingItem* item = [table itemAtRow:row];
        if(![item isResolved]){
            @try{
				[item resolve];
				[table reloadItem:item];
				if([item isResolved]){
					NSString* typeId = [item objectTypeId];
					[messageField setStringValue:typeId?typeId:@"Undefined"];
					[resolveButton setEnabled:NO];
				}
            }@catch(NSException *localException){
				[self presentException:localException];
            }
        }
    }
}

- (void) _resolveItem:sender
{
    [resolveButton performClick:self];
}

- (BOOL) acceptConnections:(NSArray*)array;
{
    BOOL ok = NO;
    @try{
		if(![self isConnected]){
			NSEnumerator* en = [array objectEnumerator];
			id obj;
			while(obj = [en nextObject]){
				if(![obj interface])
					[obj narrowInterface];
				if([[obj interface] adoptsInterface:_interface]){
					NSString* hostName = [obj host];
					unsigned short port = [[(CORBAObject*)obj port]unsignedShortValue];
					IDLGIOPVersion* version = [obj iiopVersion];
					[_interface setInterfaceConnection:obj];
					[iorView setConnection:obj];
				//    [[NSNotificationCenter defaultCenter]postNotificationName:VXUpdateObject object:nil];
					[hostField setStringValue:hostName?hostName:@""];
					[portField setIntValue:port];
					[giopPopUp selectItemAtIndex:[version minorVersion]];
					[self performSelector:@selector(listEntries) withObject:nil afterDelay:0.0];
					ok = YES;
				}
			}
		}
    }@catch(NSException *localException){
		NSLog(@"%@, %@\n",[localException name], [localException reason]);
		[_interface setInterfaceConnection:nil];
		ok = NO;
		[self presentException:localException];
    }
    return ok;
}

- (void) presentException:(NSException*)exception
{
    NSDictionary* info = [exception userInfo];
    NSString* typeId = [info objectForKey:IDLExceptionIDKey];
    IDLException* excp = nil;
    IDLExceptionController* ctrl;
    if(typeId)
        excp = (IDLException*)[[IDLDefinitionScope rootScope] definitionForTypeId:typeId];
    ctrl = [[IDLExceptionController alloc]initWithException:exception definition:excp];
    [ctrl showExceptionFor:self window:window];
}

- (void) exceptionSheetEnded:ctrl
{
    [ctrl release];
}

- (void) setScope
{
    [[IDLDefinitionScope rootScope]set];
}

- (void)controlTextDidEndEditing:(NSNotification *)notification
{
    if([notification object] ==  listCountField){
        int cnt = [listCountField intValue];
        if(cnt <= 0){
            cnt = 100;
            [listCountField setIntValue:cnt];
        }
        [[NSUserDefaults standardUserDefaults]setInteger:cnt
            forKey:@"nameServiceFetchCount"];
    }
}

@end

// Used in NamingService window
@interface VXFlippedView:NSView
{
}
@end
@implementation VXFlippedView

- (BOOL) isFlipped
{
    return YES;
}

@end

@implementation NSOutlineView(IDLNameService)
- connection
{
    int row = [self selectedRow];
    if(row >=0){
        BindingItem* item = [self itemAtRow:row];
        if([item respondsToSelector:@selector(connection)])
            return [item connection];
        else
            return nil;
    }else
        return nil;
}

@end

static NSString* HostAddress(NSHost* host)
{
	NSArray* array = [host addresses];
	NSEnumerator* en = [array objectEnumerator];
	NSString* addrString;
	while(addrString = [en nextObject]){
		if([addrString rangeOfString:@":"].length > 0)
			continue;
		if(![addrString isEqualToString:@"127.0.0.1"])
			break;
	}
	return addrString?addrString:[host address];
}

static NSString* VXHostAddress(NSHost* host)
{ 
	if(host)
		return HostAddress(host);
	else
		return @"127.0.0.1";
}
