/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of IDLBrowser.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#import "TiledBox.h"
#import <Foundation/Foundation.h>

@implementation TiledBox

static NSImage* _defaultTile = nil;
static NSMutableDictionary* _tileTable = nil;
static double numFrames = 24;
static NSPoint _defaultSpeed = {-16.25,-22.67};

+ (void)initialize
{
    if([TiledBox class] == self){
        NSBundle* bundle = [NSBundle bundleForClass:self];
        NSString* path = [bundle pathForResource:@"BoxTile" ofType:@"tiff"];
        if(path)
            _defaultTile = [[NSImage alloc]initWithContentsOfFile:path];
        _tileTable = [[NSMutableDictionary alloc]init];
    }
}

static void _drawBezel(NSRect r,NSColor* clr0,NSColor* clr1,NSColor* clr2,NSColor* clr3)
{
}

void TileViewRect(id self, NSRect rect, NSImage* image, NSPoint displacement, BOOL reconToWin)
{
    NSRect r = {{0,0},{0,0}};
    NSRect bounds = [self bounds];
    float x0;

    r.size = [image size];
    if(reconToWin){
        NSRect cr = [[[self window]contentView]frame];
        r = [self convertRect:r fromView:nil];
        r.origin.y = NSMaxY(cr);
        r.origin.x = NSMinX(cr);
    }else{
        r.size = [self convertSize:r.size fromView:nil];
        r.origin = bounds.origin;
    }
    r.origin.x += displacement.x;
    r.origin.y += displacement.y;
    while(r.origin.x > rect.origin.x) r.origin.x -= r.size.width;
    while(r.origin.y > rect.origin.y) r.origin.y -= r.size.height;
    while(NSMaxX(r) <= NSMinX(rect)) r.origin.x += r.size.width;
    while(NSMaxY(r) <= NSMinY(rect)) r.origin.y += r.size.height;

    x0 = r.origin.x;
    do{
        do{
            NSRect b = rect;
            NSRect r1 = r;
            NSPoint p;
            r1 = [self convertRect:r1 toView:nil];
            b = [self convertRect:b toView:nil];
            b = NSIntersectionRect(r1,b);
            p = b.origin;
            b.origin.x -= r1.origin.x;
            b.origin.y -= r1.origin.y;
            p = [self convertPoint:p fromView:nil];
            [image compositeToPoint:p fromRect:b operation:NSCompositeSourceOver];
            r.origin.x += r.size.width;
        }while(r.origin.x < NSMaxX(rect));
        r.origin.x = x0;
        r.origin.y += r.size.height;
    }while(r.origin.y < NSMaxY(rect));
}

- (void) tileRect:(NSRect)rect image:(NSImage*)tile
{
    TileViewRect(self,rect,tile,p,YES);
}
- (void) setTile:(NSImage*)image
{
    [_tile autorelease];
    _tile = [image retain];
}

- (NSImage*) getTile
{
    if(_tile)
        return _tile;
    else
        return [[self superview]getTile];
}

- (void) setSpeed:(NSPoint)moveSpeed
{
    speed.x = moveSpeed.x/numFrames;
    speed.y = moveSpeed.y/numFrames;
}

- (NSPoint)speed
{
    NSPoint s = {
        speed.x*numFrames,
        speed.y*numFrames
    };
    return s;
}
- (void) _windowDidBecomeKeyWindow:(NSNotification*)notification
{
    [self start];
}

- (void) _windowDidResignKeyWindow:(NSNotification*)notification
{
    [self stop];
}
- (void) _registerObserver
{
    if([self window]){
        [[NSNotificationCenter defaultCenter]addObserver:self
                                            selector:@selector(_windowDidBecomeKeyWindow:)
                                                name:NSWindowDidBecomeKeyNotification
                                              object:[self window]];
        [[NSNotificationCenter defaultCenter]addObserver:self
                                            selector:@selector(_windowDidResignKeyWindow:)
                                                    name:NSWindowDidResignKeyNotification
                                              object:[self window]];
    }
}

- initWithFrame:(NSRect)rect
{
    NSArray* types = [NSArray arrayWithObject:NSFilenamesPboardType];
    [super initWithFrame:rect];
    displacement.x = displacement.y = 0;
    p.x = p.y = 0;
    [self registerForDraggedTypes:types];
    [self setTile:_defaultTile];
    [self setSpeed:_defaultSpeed];
    return self;
}

- (void) awakeFromNib 
{
    NSArray* types = [NSArray arrayWithObject:NSFilenamesPboardType];
    if([[self superclass]instancesRespondToSelector:_cmd])
        [super awakeFromNib];
    [self registerForDraggedTypes:types];
    [self _registerObserver];
    displacement.x = displacement.y = 0;
    [self setSpeed:_defaultSpeed];
    p.x = p.y = 0;
    [self setTile:_defaultTile];
}

- (void) dealloc
{
    NSString* key = [NSString stringWithFormat:@"%x",self];
    [_tileTable removeObjectForKey:key];
    [[NSNotificationCenter defaultCenter]removeObserver:self];
    [super dealloc];
}

- (BOOL) isActive
{
    return _active;
}
- (void) moveTile
{
    NSImage* tile = [self getTile];
    NSSize size = {0,0};
    NSPoint p0 = p;
    NSAutoreleasePool* pool = [[NSAutoreleasePool alloc]init];
    if(tile){
        [[self window]disableFlushWindow];
        size = [tile size];
        displacement.x += speed.x;
        displacement.y += speed.y;
        if(displacement.x > size.width)
        displacement.x -= size.width;
        else
            if(displacement.x < -size.width)
                displacement.x += size.width;
        if(displacement.y > size.height)
            displacement.y -= size.height;
        else
            if(displacement.y < -size.height)
                displacement.y += size.height;
        p.x = floor(displacement.x);
        p.y = floor(displacement.y);
        if(p.x >= p0.x+1 || p.x <= p0.x-1 || p.y >= p0.y+1 || p.y <= p0.y-1)
            [self display];
        [[self window] enableFlushWindow];
        [[self window] flushWindow];
        [[NSGraphicsContext currentContext]flushGraphics];
    }
    [pool release];
}

- (void) moveTile:(NSTimer*)timer
{
    [self moveTile];
}


- (void) start
{
    if(timer == nil){
        timer = [[NSTimer scheduledTimerWithTimeInterval:1.0/numFrames
                                                  target:self
                                                selector:@selector(moveTile:)
                                                userInfo:nil
                                                 repeats:YES]retain];
    }
    _active = YES;
}

- (void) stop
{
    [timer invalidate];
    [timer release];
    timer = nil;
    _active = NO;
}

- (void) drawRect:(NSRect)rect
{
    id tile = [self getTile];
    if(tile){
        [self tileRect:[self bounds] image:tile];
        switch([self borderType]){
            case NSLineBorder:
                [[NSColor blackColor]set];
                NSFrameRect([self borderRect]);
                break;
            case NSBezelBorder:
                _drawBezel([self borderRect],
                           [NSColor darkGrayColor],
                           [NSColor blackColor],
                           [NSColor whiteColor],
                           [NSColor lightGrayColor]);
                break;
            case NSGrooveBorder:
                _drawBezel([self borderRect],
                           [NSColor darkGrayColor],
                           [NSColor whiteColor],
                           [NSColor whiteColor],
                           [NSColor darkGrayColor]);
                break;
            default:
                break;
        }
        if([self titlePosition] != NSNoTitle){
            [self tileRect:[self titleRect] image:tile];
            [[self titleCell] drawInteriorWithFrame:[self titleRect] inView:self];
        }
    }else
        [super drawRect:rect];
}

@end

@implementation TiledBox(Dragging)

static unsigned int op = 0;
static NSImage* _draggedImage = nil;

- (unsigned)draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard* pboard = [sender draggingPasteboard];
    NSArray* types = [NSArray arrayWithObject:NSFilenamesPboardType];
    NSString* type = [pboard availableTypeFromArray:types];
    if(type){
        NSArray* files = [pboard propertyListForType:type];
        _draggedImage = [[NSImage alloc]initWithContentsOfFile:[files objectAtIndex:0]];
        if(_draggedImage)
            op = NSDragOperationGeneric;
        else
            op = 0;
    }
    return op;
}

- (unsigned)draggingUpdated:(id <NSDraggingInfo>)sender
{
    return op;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    [_draggedImage release];
    _draggedImage = nil;
    op = 0;
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    return YES;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    if(_draggedImage){
        [self setTile:[_draggedImage autorelease]];
        return YES;
    }else
        return NO;
}

- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
    [self display];
}


@end

@implementation NSView(TiledBox)
- (NSImage*)getTile
{
    return [[self superview] getTile];
}
@end
