
#import "ValueTestClient.h"
#import "ValueTest.h"
#import "WTree.h"
#import <ADORB/ADORB.h>
#import "NamingContextProto.h"
/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

@implementation ValueTestClient

// Programmatically set object class for 'WTree'
- (void) setup
{
    IDLValueType* value = (IDLValueType*)[[ORB defaultORB]definitionNamed:@"vtest/WTree"];
    [value setObjectClassName:@"WTree"];
}

- init
{
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    CORBAObject* nameService = [ORB resolveInitialReferences:@"NameService"];
    self = [super init];
    @try{
		if(nameService){
			NSLog(@"Obtaining server from Name Service");
			server = [[nameService resolve:[IDLNameComponent nameComponentsForPath:@"ORBValueTest"]]retain];
		}else{
			NSLog(@"Obtaining server from IOR");
			NSString* iorPath = [defaults stringForKey:@"ior"];
			NSString* ior = [[NSString alloc]initWithContentsOfFile:iorPath];
			server = [[CORBAObject objectWithIOR:ior]retain];
		}
    }@catch(NSException *localException){
		NSLog(@"%@: %@\n", [localException name], [localException reason]);
    }
    [self setup];
    return self;
}

- (void) run
{
    id object, array;
    @try{
    
		object = [server getBox:@"box" width:10 height:10];
		NSLog(@"getBox: %@", [object description]);
		
		object = [server getNode:@"node"];
		NSLog(@"getNode: %@", [object description]);
		
		object = [server getLink:@"link" weight:100];
		NSLog(@"getLink: %@", [object description]);
		
		object = [server getTree];
		NSLog(@"getTree: %@", [object description]);
		
		[server getMulti:&object boxes:&array];
		NSLog(@"getMulti: %@\nboxes=%@", [object description],[array description]);

		id node = [server valueForAttribute:@"node"];
		NSLog(@"attribute node: %@", node);
		
		id link = [server valueForAttribute:@"link"];
		[link setValue:@"new" forKey:@"name"];
		[server setValue:link forAttribute:@"link"];
		id link1 = [server valueForAttribute:@"link"];
		NSAssert([link1 isEqual:link], @"failed");
		
		object = [server getAny1];
		NSLog(@"getAny1: %@", [object description]);

		object = [server getAny2];
		NSLog(@"getAny2: %@", [object description]);

		object = [server getAny3];
		NSLog(@"getAny3: %@", [object description]);

		object = [server getAny4];
		NSLog(@"getAny4: %@", [object description]);
    
    }@catch(NSException *localException){
		NSLog(@"%@, %@", [localException name], [localException reason]);
    
    }
}

@end

int main (int argc, const char *argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
    ValueTestClient* client = [[ValueTestClient alloc]init];
    [client run];
    [client release];

    [pool release];
    exit(0);       // insure the process exit status is 0
    return 0;      // ...and make main fit the ANSI spec.
}

