/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLArgument.h"
#import "IDLDefinition.h"
#import <expr.tab.h>

@implementation IDLArgument

- initWithName:(NSString*)name type:(IDLDefinition*)type mode:(int)mode
{
	self = [super init];
	_name = [name copy];
	_type = [type retain];
	_mode = mode;
	if(mode != IN && mode != OUT && mode != INOUT)
		[NSException raise:@"InvalidArgument" format:@"invalid argument mode %d", mode];
	return self;
}

+ argumentWithName:(NSString*)name type:(IDLDefinition*)type mode:(int)mode
{
	return [[[self allocWithZone:[IDLDefinition zone]]initWithName:name type:type mode:mode]autorelease];
}

- (NSString*) name
{
	return _name;
}

- (IDLDefinition*)definition
{
	return _type;
}

- (int) mode
{
	return _mode;
}

- (NSString*) stringMode
{
    switch(_mode){
        case IN: return @"in";
        case OUT: return @"out";
        case INOUT: return @"inout";
        default: return @"";
    }
}

- (BOOL) isInMode
{
    switch(_mode){
        case IN: 
		case INOUT: return YES;
        case OUT:
        default: return NO;
    }
}

- (BOOL) isOutMode
{
    switch(_mode){
        default:
        case IN: return NO;
		case INOUT:
        case OUT: return YES;
    }
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_className forKey:@"className"];
		[coder encodeInt:_mode forKey:@"mode"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_name];
		[coder encodeObject:_className];
		[coder encodeValueOfObjCType:@encode(int) at:&_mode];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_className = [[coder decodeObjectForKey:@"className"]retain];
		_mode = [coder decodeIntForKey:@"mode"];
	}else{
		_type = [[coder decodeObject]retain];
		_name = [[coder decodeObject]retain];
		_className = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(int) at:&_mode];
	}
	return self;
}

@end
