/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLOctetArray.h"
#import "IDLDefinition_Private.h"
#import "IDLEncapsulation.h"

@implementation IDLArray(CORBACoding)

- (void) encodeObject:obj withCoder:(id <CORBACoder>)inv
{
    int n = 0;
    for(n=0;n<_size;n++)
        [_type encodeObject:[obj objectAtIndex:n] withCoder:inv];
}

- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)inv
{
    int n;
    NSMutableArray* array = [NSMutableArray array];
    for(n=0;n<_size;n++){
        id item = nil;
        [_type decodeObject:&item withCoder:inv];
        if(item)
            [array addObject:item];
        else
            [array addObject:[NSNull null]];
    }
    *obj = array;
}

- (const char*) objCType
{
    return @encode(id);
}
- (unsigned) argLength
{
    return sizeof(id);
}
- (unsigned) returnValueLength
{
    return sizeof(id);
}

- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n
{
    // Array is mapped to NSArray (NSData for Octet Array)
    id array = nil;
    [inv getArgument:&array atIndex:n];
    return array;
}

- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr
{
    id array = *((id*)ptr);
    return array;
}

- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n
{
    id _val = value;
    [inv setArgument:&_val atIndex:n];
}

- (void) setValue:value ofType:(const char*)argType forPointer:(void*)ptr
{
    *((id*)ptr) = value;
}
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv
{
    id _val = value;
    [inv setReturnValue:&_val];
}

- returnValueFromInvocation:(NSInvocation*)inv
{
    id _val = nil;
    [inv getReturnValue:&_val];
    return _val;
}

- (void) _encodeWithCORBACoder:(id <CORBACoder>)coder
{
    [_type encodeWithCORBACoder:coder];
    [coder encodeULong:_size];
}
- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    IDLEncapsulation* stream = [IDLEncapsulation encapsulation];
    [coder encodeLong:[self kind]];
    [stream setParentStream:coder location:[coder location]];
    // Don't use indirection for array
    //[stream registerLocation:-2*sizeof(long) forDefinition:self];
    [self _encodeWithCORBACoder:stream];
    [coder encodeData:[stream data]];
}

@end
