/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLFunction_Private.h"
#import "IDLArgument.h"
#import "IDLInterface_Private.h"
#import "IDLValueType.h"
#import <expr.tab.h>

@implementation IDLFunction

static NSLock* _lock = nil;

+ (void) becomeMultiThreaded:(NSNotification*)notification
{
    _lock = [[NSLock alloc]init];
}

+ (void) initialize
{
    if([IDLFunction class] == self){
        if([NSThread isMultiThreaded])
            [self becomeMultiThreaded:nil];
        else
            [[NSNotificationCenter defaultCenter] addObserver:self
                selector:@selector(becomeMultiThreaded:)
                name:NSWillBecomeMultiThreadedNotification object:nil];
    }
}

- (NSLock*) getLock
{
    return _lock;
}

- (void) setIsFactory:(BOOL)flag
{
    _isFactory = flag?YES:NO;
}

- (BOOL) isFactory
{
    return _isFactory;
}

- (void) setExceptionList:(NSArray*)list
{
    _exceptionList = [list retain];
}

- (void) setContextList:(NSArray*)list
{
    _contextList = [list retain];
}

- initWithName:(NSString*)aName returning:retType args:(NSArray*)theArgs
{
    self = [super init];
    _name = [aName copy];
    _returnType = [retType retain];
    _args = [theArgs retain];
    return self;
}
    
+ function:(NSString*)aName returning:retType args:(NSArray*)theArgs
{
    return [[[self allocWithZone:[self zone]]initWithName:aName returning:retType args:theArgs]autorelease];
}

- (void) dealloc
{
    [_args release];
    [_name release];
    [_returnType release];
    [_exceptionList release];
    [_contextList release];
    [_selector release];
    [_objCTypeString release];
    if(_data)
        NSZoneFree([self zone],_data);
    if(_signature)
        NSZoneFree([self zone],_signature);
    [super dealloc];
}

- (NSString*)name
{
    return _name;
}

- (NSString*) description
{
    return [NSString stringWithFormat:@"%@<0x%x>",
        NSStringFromClass([self class]), self];
}

- (void) setInterfaceObject:(IDLInterface*) anObj
{
    _interfaceObject = anObj;
}

- (IDLInterface*)interfaceObject
{
    return _interfaceObject;
}


- (IDLDefinition*) returnType
{
    return _returnType;
}

- (BOOL) isOneWay
{
    return [_returnType isOneWay];
}

- (NSArray*) arguments
{
    return _args;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_args forKey:@"args"];
		[coder encodeObject:_returnType forKey:@"return"];
		[coder encodeObject:_interfaceObject forKey:@"interface"];
		[coder encodeObject:_exceptionList forKey:@"exceptions"];
		[coder encodeObject:_contextList forKey:@"context"];
		[coder encodeBool:_isFactory forKey:@"factory"];
	}else{
		[coder encodeObject:_name];
		[coder encodeObject:_args];
		[coder encodeObject:_returnType];
		[coder encodeObject:_interfaceObject];
		[coder encodeObject:_exceptionList];
		[coder encodeObject:_contextList];
		[coder encodeValueOfObjCType:@encode(BOOL) at:&_isFactory];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_args = [[coder decodeObjectForKey:@"args"]retain];
		_returnType = [[coder decodeObjectForKey:@"return"]retain];
		_interfaceObject = [[coder decodeObjectForKey:@"interface"]retain];
		_exceptionList = [[coder decodeObjectForKey:@"exeptions"]retain];
		_contextList = [[coder decodeObjectForKey:@"context"]retain];
		_isFactory = [coder decodeBoolForKey:@"factory"];
	}else{
		_name = [[coder decodeObject]retain];
		_args = [[coder decodeObject]retain];
		_returnType = [[coder decodeObject]retain];
		_interfaceObject = [[coder decodeObject]retain];
		_exceptionList = [[coder decodeObject]retain];
		_contextList = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(BOOL) at:&_isFactory];
	}
	return self;
}

@end
