/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>
#import "IDLDefinition_Private.h"
#import "IDLSequence.h"

@implementation IDLSequence

- setDefType:(IDLDefinition*)aType
{
    [type autorelease];
    type = [aType retain];
    return self;
}

- (IDLDefinition*) sequenceType
{
    return type;
}

- (NSString*) name
{
    if(_frameCount > 0)
        return [NSString stringWithFormat:@"sequence <%@,%d>",[type name],_frameCount];
    else
        return [NSString stringWithFormat:@"sequence <%@>",[type name]];
}

- initWithType:(IDLDefinition*)aType count:(unsigned)count
{
    self = [super init];
    type = [aType retain];
    _frameCount = count;
    return self;
}

+ sequenceWithType:(IDLDefinition*)aType count:(unsigned)count
{
    id class = [aType sequenceClass];
    return [[[class allocWithZone:[self zone]]initWithType:aType count:count]autorelease];
}

+ sequenceWithType:(IDLDefinition*)aType
{
    return [self sequenceWithType:aType count:0];
}

- (void) dealloc
{
    [type release];
    [super dealloc];
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:type forKey:@"type"];
		[coder encodeInt:_frameCount forKey:@"count"];
	}else{
		[coder encodeObject:type];
		[coder encodeValueOfObjCType:@encode(unsigned) at:&_frameCount];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		type = [[coder decodeObjectForKey:@"type"]retain];
		_frameCount = [coder decodeIntForKey:@"count"];
	}else{
		type = [[coder decodeObject]retain];
		[coder decodeValueOfObjCType:@encode(unsigned) at:&_frameCount];
	}
	return self;
}
- (id)awakeAfterUsingCoder:(NSCoder *)aDecoder
{
	return self;
}

- (long) kind
{
    return tk_sequence;
}

@end

