/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLStructMember.h"
#import "IDLDefinition.h"

@implementation IDLStructMember

- initWithName:(NSString*)name type:(IDLDefinition*)type
{
	self = [super init];
	_type = [type retain];
	_name = [name copy];
	return self;
}

- (void) dealloc
{
	[_type release];
	[_name release];
	[_className release];
	[super dealloc];
}

+ structMemberWithName:(NSString*)name ofType:(IDLDefinition*)type
{
	return [[[self alloc]initWithName:name type:type]autorelease];
}

- (NSString*)name
{
	return _name;
}

- (IDLDefinition*)type
{
	return _type;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_className forKey:@"className"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_name];
		[coder encodeObject:_className];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_className = [[coder decodeObjectForKey:@"className"]retain];
	}else{
		_type = [[coder decodeObject]retain];
		_name = [[coder decodeObject]retain];
		_className = [[coder decodeObject]retain];
	}
	return self;
}

- (BOOL) isEqual:(IDLStructMember*)member
{
	return member == self || [member isMemberOfClass:[self class]] && [member->_name isEqualToString:_name] &&
		[member->_type isEqual:_type];
}

@end
