/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLUnion.h"
#import "IDLSwitchCase.h"
#import "IDLDefinition_Private.h"

@implementation IDLUnion

- initWithName:(NSString*)name switchType:(IDLDefinition*)type
{
    self = [super init];
    _name = [name retain];
    _type = [type retain];
    _caseList = [[NSMutableArray alloc]init];
    return self;
}

+ union:(NSString*)name ofType:(IDLDefinition*)type
{
    return [[[self allocWithZone:[self zone]]initWithName:name switchType:type]autorelease];
}

- (void) dealloc
{
    [_name release];
    [_type release];
    [_caseList release];
    [super dealloc];
}

- (IDLDefinition*) switchType
{
    return _type;
}

- (NSString*) name
{
    return _name;
}

- (void) addCase:(IDLSwitchCase*)caseObject
{
    if([caseObject isDefault])
        _defaultCase = [caseObject retain];
    else
        [_caseList addObject:caseObject];

}

- (IDLSwitchCase*) caseForValue:value
{
	if(value != nil){
		NSEnumerator* en = [_caseList objectEnumerator];
		IDLSwitchCase* obj;
		while(obj = [en nextObject]){
			if([[obj caseValue] isEqual:value])
				return obj;
		}
	}
    return _defaultCase;
}

- (NSString*) description
{
    return [[super description]stringByAppendingFormat:@"union %@(%@) %@\n%@",
        [self name],[self switchType],
        [_caseList description],_defaultCase?[_defaultCase description]:@""];
}

- (void) setObjectClassName:(NSString*)string
{
    if(string != _className){
        [_className release];
        _className = [string copy];
    }
}
- (NSString*) objectClassName
{
    return _className;
}

- (long) kind
{
    return tk_union;
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	[super encodeWithCoder:coder];
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_name forKey:@"name"];
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_caseList forKey:@"cases"];
		[coder encodeObject:_defaultCase forKey:@"defaultCase"];
		[coder encodeObject:_className forKey:@"className"];
	}else{
		[coder encodeObject:_name];
		[coder encodeObject:_type];
		[coder encodeObject:_caseList];
		[coder encodeObject:_defaultCase];
		[coder encodeObject:_className];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super initWithCoder:coder];
	if([coder allowsKeyedCoding]){
		_name = [[coder decodeObjectForKey:@"name"]retain];
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_caseList = [[coder decodeObjectForKey:@"cases"]retain];
		_defaultCase = [[coder decodeObjectForKey:@"defaultCase"]retain];
		_className = [[coder decodeObjectForKey:@"className"]retain];
	}else{
		_name = [[coder decodeObject]retain];
		_type = [[coder decodeObject]retain];
		_caseList = [[coder decodeObject]retain];
		_defaultCase = [[coder decodeObject]retain];
		_className = [[coder decodeObject]retain];
	}
	return self;
}

@end
