/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLUnion.h"
#import "IDLSwitchCase.h"
#import "IDLDefinition_Private.h"
#import "IDLEncapsulation.h"
#import "IDLLink.h"
#import "IDLDefinitionScope.h"

@implementation IDLUnion(CORBACoding)

NSString *IDLUnionDescriptorKey = @"descriptor";
NSString *IDLUnionDataKey = @"value";
NSString *VXUnionDescriptorKey = @"descriptor";
NSString *VXUnionDataKey = @"value";

- (IDLDefinition*)_getTypeForDescriptor:value
{
    IDLSwitchCase* caseObj = [self caseForValue:value];
    return [caseObj caseType];
}

- (void) encodeObject:obj withCoder:(id <CORBACoder>)inv
{
    id descriptorValue = [obj valueForKey:IDLUnionDescriptorKey];
    id dataValue = [obj valueForKey:IDLUnionDataKey];
    IDLDefinition* dataDef = [self _getTypeForDescriptor:descriptorValue];
    [_type encodeObject:descriptorValue withCoder:inv];
    [dataDef encodeObject:dataValue withCoder:inv];
}

- (void) decodeObject:(id*)obj withCoder:(id <CORBADecoder>)inv
{
    id descriptorValue = nil;
    id dataValue = nil;
    IDLDefinition* dataDef;
    id objClass;
    
    [_type decodeObject:&descriptorValue withCoder:inv];
    dataDef = [self _getTypeForDescriptor:descriptorValue];
    [dataDef decodeObject:&dataValue withCoder:inv];
   
     objClass = _className?NSClassFromString(_className):[NSMutableDictionary class];
    if(!objClass)
        objClass = [NSMutableDictionary class];
    *obj = [[[objClass alloc]init]autorelease];
    [*obj setValue:descriptorValue forKey:IDLUnionDescriptorKey];
    [*obj setValue:dataValue forKey:IDLUnionDataKey];
}

- (const char*) objCType
{
    return @encode(id);
}
- (unsigned) argLength
{
    return sizeof(id);
}
- (unsigned) returnValueLength
{
    return sizeof(id);
}

- argumentObjectFromInvocation:(NSInvocation*)inv atIndex:(int)n
{
    // Struct is mapped to NSDictionary
    id dict = nil;
    [inv getArgument:&dict atIndex:n];
    return dict;
}

- argumentObjectOfType:(const char*)argType fromPointer:(void*)ptr
{
    id dict = *((id*)ptr);
    return dict;
}

- (void) setValue:value ofType:(const char*)argType forPointer:(void*)ptr
{
    *((id*)ptr) = value;
}
- (void) setValue:value forInvocation:(NSInvocation*)inv atIndex:(int)n
{
    id _val = value;
    [inv setArgument:&_val atIndex:n];
}
- (void) setReturnValue:value forInvocation:(NSInvocation*)inv
{
    id _val = value;
    [inv setReturnValue:&_val];
}

- returnValueFromInvocation:(NSInvocation*)inv
{
    id _val = nil;
    [inv getReturnValue:&_val];
    return _val;
}

- (void) _encodeWithCORBACoder:(id <CORBACoder>)coder
{
    NSEnumerator* en = [_caseList objectEnumerator];
    int count = [_caseList count];
    IDLSwitchCase* obj;
    [coder encodeString:[self typeId]];
    [coder encodeString:[self name]];
    [_type encodeWithCORBACoder:coder];
    if(_defaultCase)
        [coder encodeLong:count++];
    else
        [coder encodeLong:-1];
    [coder encodeLong:count];
    while(obj = [en nextObject])
        [obj encodeWithCORBACoder:coder];
        
    if(_defaultCase)
        [_defaultCase encodeWithCORBACoder:coder];
}
- (void) encodeWithCORBACoder:(id <CORBACoder>)coder
{
    long location = [coder locationForDefinition:self];
    if(location != 0 && [IDLDefinition allowsIndirection]) // already encoded -> encode indirection
        [coder encodeIndirection:location];
    else{
        IDLEncapsulation* stream = [IDLEncapsulation encapsulation];
        [coder encodeLong:[self kind]];
        [stream setParentStream:coder location:[coder location]];
        [stream registerLocation:-2*sizeof(long) forDefinition:self];
        [self _encodeWithCORBACoder:stream];
        [coder encodeData:[stream data]];
    }
}


- _initWithCoder:(id<CORBADecoder>)coder
{
    IDLDefinition* type;
    NSString *str;
    NSString* typeId;
    long cnt = 0;
    long index, n = 0;
    
    [coder decodeString:&typeId];
    [coder decodeString:&str];
    type = [IDLDefinition definitionFromCoder:coder];
    self = [self initWithName:str switchType:type];
    [self setTypeId:typeId];
    
    [coder decodeLong:&index];
    [coder decodeLong:&cnt];
    
    while(cnt-- > 0){
        NSNumber* value = nil;
        NSString* memberName = nil;
        IDLDefinition* memberType;
        [type decodeObject:&value withCoder:coder];
        [coder decodeString:&memberName];
        memberType = [IDLDefinition definitionFromCoder:coder];
        if(n == index)
            value = nil; // Default case
        [self addCase:[IDLSwitchCase switchCase:value dataType:memberType switchType:type name:memberName]];
        n += 1;
    }
    return self;
}

+ _definitionFromCoder:(IDLEncapsulation*)coder
{
    IDLLink* link = [IDLLink linkWithDefinition:nil];
    IDLUnion* obj = [[IDLUnion alloc]autorelease];
    IDLUnion* existingObject = nil;

    [coder registerDefinition:(IDLDefinition*)link forLocation:-2*sizeof(long)];

    obj = [obj _initWithCoder:coder];
    existingObject = (IDLUnion*)[[IDLDefinitionScope currentScope]definitionForTypeId:[obj typeId]];
    if([existingObject isEqual:obj])
        [link setLink:existingObject];
    else
        [link setLink:obj];
    
    return link;
}

- (BOOL) isEqual:(IDLUnion*)def
{
	return [super isEqual:def];
#warning  NOT IMPLEMENTED -isEqual:  
}

@end
