/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLValueObject.h"
#import "IDLValueType.h"

@implementation IDLValueObject

- initWithIDLType:(id<IDLDefinition>)type
{
    self = [super init];
    _type = [(NSObject*)type retain];
    return self;
}

- init
{
    return [self initWithIDLType:nil];
}

- (void) dealloc
{
    [_type release];
    [_value release];
    [super dealloc];
}

- (void) encodeWithCoder:(NSCoder*)coder
{
	if([coder allowsKeyedCoding]){
		[coder encodeObject:_type forKey:@"type"];
		[coder encodeObject:_value forKey:@"value"];
	}else{
		[coder encodeObject:_type];
		[coder encodeObject:_value];
	}
}

- initWithCoder:(NSCoder*)coder
{
	self = [super init];
	if([coder allowsKeyedCoding]){
		_type = [[coder decodeObjectForKey:@"type"]retain];
		_value = [[coder decodeObjectForKey:@"value"]retain];
	}else{
		_type = [[coder decodeObject]retain];
		_value = [[coder decodeObject]retain];
	}
	return self;
}

+ objectWithIDLType:(id<IDLDefinition>)type
{
    return [[[self alloc]initWithIDLType:type]autorelease];
}
+ objectWithValueType:(IDLValueType*)type
{
    return [[[self alloc]initWithIDLType:type]autorelease];
}

- (void) setValue:value forKey:(NSString*)key
{
    if(!_value)
        _value = [[NSMutableDictionary allocWithZone:[self zone]]init];

    [_value setValue:value forKey:key];
}

- valueForKey:(NSString*)key
{
    return [_value valueForKey:key];
}

- (IDLValueType*)valuetype
{
	IDLValueType* type = (IDLValueType*)[self idlType];
	if([type isKindOfClass:[IDLValueType class]])
		return type;
	else
		return nil;
}

- (id<IDLDefinition>)idlType
{
    return _type;
}

- (void) setIDLType:(id<IDLDefinition>)type;
{
    if(type != _type){
        [_type release];
        _type = [(NSObject*)type retain];
    }
}
- (void) setValuetype:(IDLValueType*)type
{
	[self doesNotRecognizeSelector:_cmd];
}

- (void) setValue:value
{
    if(value != _value){
        [_value release];
        _value = [value retain];
    }
}

- (void) setWrappedValue:value
{
    if(value != _value){
        [_value release];
        _value = [value retain];
    }
}

- value
{
    return _value;
}

- wrappedValue
{
	return _value;
}

- (NSString*) stringValue
{
    return [NSString stringWithFormat:@"<0x%x>", self];
}

- (NSMethodSignature*) methodSignatureForSelector:(SEL)selector
{
    return [_value methodSignatureForSelector:selector];
}

- (void) forwardInvocation:(NSInvocation*)invocation
{
    [invocation invokeWithTarget:_value];
}

@end
