/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import "IDLProfileBody.h"
#import "IDLGIOPVersion.h"
#import "IDLTaggedComponent.h"
#import "CORBAObject_Private.h"

#define DEFAULT_PORT 1050

@implementation IDLProfileBody

- initWith:version host:host port:(unsigned)port objectID:(NSData*)objID
{
    self = [super init];
    _iiop_version = [version retain];
    _port = port;
    _host = [host retain];
    _object_key = [objID retain];
    return self;
}

- initWithCorbaloc:(NSString*)corbaloc
{
// corbaloc:[iiop]:[[1.2@]<host>[:<port>]][/<object-key>]
// arg = [iiop]:[[1.2@]<host>[:<port>]][/<object-key>]
    NSRange r = [corbaloc rangeOfString:@"/"];
    NSString* key = @"NameService";
    NSArray* strings = nil;
    NSString* proto = @"iiop";
    unsigned short port = DEFAULT_PORT;
    NSString* hostname;
    IDLGIOPVersion* version = nil;
    if(r.length > 0){
        key = [corbaloc substringFromIndex:r.location  + r.length];
        strings = [[corbaloc substringToIndex:r.location]componentsSeparatedByString:@":"];
    }else
        strings = [corbaloc componentsSeparatedByString:@":"];
    
    proto = [strings objectAtIndex:0];
    if([proto length] == 0 || [proto isEqualToString:@"iiop"]){
        NSArray* array = [[strings objectAtIndex:1]componentsSeparatedByString:@"@"];
        if([array count] == 1){
            hostname = [array objectAtIndex:0];
            version = [[[IDLGIOPVersion alloc]init]autorelease];
        }else{
            NSArray* versionArray = [[array objectAtIndex:0]componentsSeparatedByString:@"."];
            version = [[[IDLGIOPVersion alloc]initVersion:
                [[versionArray objectAtIndex:0]intValue]:
                [[versionArray objectAtIndex:1]intValue]]autorelease];
            hostname = [array objectAtIndex:1];
        }
        if([strings count] > 2)
            port = [[strings objectAtIndex:2]intValue];

		if([hostname length] == 0)
			hostname = @"localhost";

        return [self initWith:version host:hostname port:port 
            objectID:[key dataUsingEncoding:NSASCIIStringEncoding]];
        
    }else
    if([proto isEqualToString:@"rir"]){
        [NSException raise:CORBAInvalidArgumentException
            format:@"invalid 'corbaloc' or 'corbaname' format,  protocol '%@' is not supported", proto];
    }else
        [NSException raise:CORBAInvalidArgumentException
            format:@"invalid 'corbaloc' or 'corbaname' format, unknow protocol '%@'", proto];
    
    return nil;
}

+ profileWithCorbaloc:(NSString*)corbaloc
{
    return [[[self alloc]initWithCorbaloc:corbaloc]autorelease];
}

- (void) dealloc
{
    [_iiop_version release];
    [_host release];
    [_object_key release];
    [_components release];
    [super dealloc];
}

+ profileWith:version host:host port:(unsigned)port objectID:(NSData*)objID
{
    return [[[self alloc]initWith:version host:host port:port objectID:objID]autorelease];
}

- (NSString*) hostName
{
    return _host;
}

- (NSString*) host
{
	if(_host)
		return _host;
	else{
		NSString* host = [CORBAObject localHostAddress];
		return host? host : @"127.0.0.1";
	}
}

- (void) setHost:(NSString*)host
{
	[_host release];
	_host = [host retain];
}

- (unsigned short) portNumber
{
    return _port;
}

- (NSData*) objectID;
{
    return _object_key;
}

- (IDLGIOPVersion*)iiopVersion
{
    return _iiop_version;
}

- (void) setPort:(NSNumber*)port
{
    _port = [port unsignedShortValue];
}

- (NSNumber*)port
{
    return [NSNumber numberWithUnsignedShort:_port];
}

- (NSArray*) components
{
    return _components? [NSArray arrayWithArray:_components] : [NSArray array];
}

- (void) addComponent:(IDLTaggedComponent*)component
{
    if(!_components)
        _components = [[NSMutableArray alloc]init];
    [_components addObject:component];
}

- (IDLTaggedComponent*) componentWithTag:(unsigned)tag
{
	NSEnumerator* en = [_components objectEnumerator];
	IDLTaggedComponent* comp;
	while(comp = [en nextObject])
		if([comp componentTag] == tag)
			break;
	return comp;
}

@end
