/*
 *  AdORB
 *  Copyright (C) 2006 Victor Ananiev
 *
 *  This file is part of ADORB.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#import <Foundation/Foundation.h>

enum _ConnectionMode {
    VXConnectionClient,
    VXConnectionServer,
    VXConnectionPassive
};

typedef struct _Conn_Flags{
    unsigned invalid:1;
    unsigned isListening:1;
    unsigned released:1;
    unsigned mode:2;
	unsigned stringEncodingSet:1;
	unsigned readState:3;
	unsigned isCodeSetsSent:1;
    unsigned reserved:22;
} VXConnectionFlags;

@class VXMessage, VXInvocation, CORBAObject, IDLServiceContext;
@class VXBuffer, VXWriteBuffer,VXMessageHeader;

@interface VXConnection : NSObject
{
    void* _inetAddr; // 'local' for passive socket; 'remote' for 'connect' and 'accepted' socket
    short _fd;	// connected socket
    NSMutableDictionary* _invocations; // Waiting invocations;
    NSMutableDictionary* _objects; // Vended objects;
	VXBuffer* _readBuffer;
	VXWriteBuffer* _writeBuffer;
    NSLock* _lock;
    NSString* _address;
    NSString* _hostname;
	NSMutableDictionary* _listenPoints;
	NSStringEncoding _stringEncoding;
	NSStringEncoding _wstringEncoding;
    VXConnectionFlags _flags;
}

+ connectToHostNamed:(NSString*)host port:(int)port timeout:(NSTimeInterval)timeout error:(NSError**)error;
+ connection; // returns the shared connection that accepts new connections

- (void) writeData:(NSData*)data;
- (void) sendMessage:message;
- (void) sendMessage:message invocation:invocation;

- (void) invalidate;
- (void) invalidateWithError:(NSError*)error;
- (BOOL) isValid;
- (void) listen;
- (int) mode;
- objectForObjectID:(NSData*)objectKey;
- (void) registerImplementationFor:(CORBAObject*)obj;
+ (void) removeObject:(CORBAObject*)obj;

+ (void) setTimeout:(NSTimeInterval)seconds;
+ (NSTimeInterval) timeout;

+ (void) run;
+ (void) resetConnections;
- (NSString*)remoteAddress;
- (NSString*)remoteHostName;
- (NSString*)hostName;
- (NSString*)address;
- (NSNumber*) port;
- (NSNumber*) localPort;
- (NSNumber*) remotePort;

- (NSDictionary*) localPoint;
- (NSDictionary*) remotePoint;
- localPointsContext;

+ (BOOL) handlesRequestsOnMainThread;
+ (void) setHandlesRequestsOnMainThread:(BOOL)flag;
+ (unsigned) maxNumberOfInvocationThreads;
+ (void) setMaxNumberOfInvocationThreads:(unsigned)num;

+ (void) enableLogging:(BOOL)flag;

- (void) setStringEncodingsFromCodeSets:(IDLServiceContext*)codeSets;
- (NSStringEncoding) wstringEncoding;
- (NSStringEncoding) stringEncoding;

- (BOOL) isStringEncodingsSet;
- (void) setStringEncodings:(NSStringEncoding)charCode:(NSStringEncoding)wcharCode;
- (BOOL) isCodeSetsSent;
- (void) setCodeSetsSent:(BOOL)flag;

- (void) addListenPoints:(NSArray*)lpArray;
+ findConnectionWithListenPointHost:(NSString*)host port:(unsigned short)port;

@end

extern NSString* VXInvalidateConnection;
extern NSString* VXAcceptedConnection;
